@@ -17,7 +17,6 @@ from .basic_types import (
     Dispatch,
     DispatchParameters,
     EventSubscriber,
-    EventSubscriptionOptions,
     FinishAction,
     FinishEvent,
     InitAction,
@@ -43,7 +42,6 @@ __all__ = (
     'Dispatch',
     'DispatchParameters',
     'EventSubscriber',
-    'EventSubscriptionOptions',
     'FinishAction',
     'FinishEvent',
     'InitAction',
